/*
    apwpost.h -- UCD interface
    Copyright 1987-1996, Macromedia, Inc. All rights reserved.

    Purpose:
    This header defines the message interface between UCDs 
    and Authorware.

    See USERCODE.WRI for full documentation of this interface.

    Revision History:
    7/30/91 - JEH: release version
    9/22/92 - DWS: added APWN_WINDOWOPEN/CLOSE msgs
    1/09/95 - PTH: added APWN_SETPROPERTY/GETPROPERTY/CLOSEWINDOW
    3/13/95 - DWS: added APWC_GET/SETVAR
                   added APWN_MOUSEMOVE/HITTEST
                   added AWPARAM struct
                   added field to APWC_POSTPB struct
                   added fields to APWN_AB struct
                   improved documentation and corrected errors
    8/07/95 - PTH: added APWC_CLOSESOUNDDRIVER
    2/02/96 - DWS: changed wparams to WPARAM and lparams to LPARAM
                   changed all WORDs and shorts to UINTs and ints
	 8/12/96 - DWS: add pack pragmas to ensure proper struct packing
*/

#ifndef _APWPOST_
#define _APWPOST_

#if defined(_WINDOWS) && !defined(RC_INVOKED)
	#if (_MSC_VER >= 900)
		#pragma pack(push)
	#endif
	#pragma pack(1)
#endif // !def RC_INVOKED


// ------------------------------------------------------------------------
//
// AWVER - Authorware version number.
//
// To exclude definitions introduced in version 3.0 (or above) 
// #define AWVER 0x200 before #including this file.
// To exclude definitions introduced in version 3.0.1 (or above) 
// #define AWVER 0x300
//
// ------------------------------------------------------------------------

#ifndef AWVER
#define AWVER 0x301
#endif


// ------------------------------------------------------------------------
//
// AWPARAM - Hidden parameter block for UCDs. 
// 
//  A pointer to this structure is passed as the first argument to a UCD
// function that specifies an 'A' as it's first argument in the resource
// description.
//
// ------------------------------------------------------------------------

#if (AWVER >= 0x300)

typedef struct {
    UINT        size;               // size of this structure
    HWND        hwnd;               // presentation window handle or NULL
    BOOL        authoring;          // in authoring system or runtime
    DWORD       reserved1;          // reserved for future expansion
    DWORD       reserved2;          // reserved for future expansion
} AWPARAM, FAR *AWPARAM_PTR;

#endif


// ------------------------------------------------------------------------
//
// APWC_POSTPB - Parameter block for APWC_POST message.
//
//  The name_ptr field was added in AW 3.0 to support named objects.
// If you recompile older UCDs that use this struct with this
// header file be sure to clear the name_ptr field.
//
// ------------------------------------------------------------------------

#define STATIC_PORT 0           // values for port field of APWC_POSTPB
#define MOTION_PORT 1
#define SCREEN_PORT 2

typedef struct {
    UINT        size;               // size of this structure
    LPSTR       dll_ptr;            // name of DLL containing object proc
    LPSTR       function_ptr;       // name of object proc (of type APW_PROC).
    RECT        rect;               // bounding rectangle of the object
    int         port;               // port to use
    LPVOID      data_ptr;           // far pointer to any private data or NULL
    UINT        data_size;          // size of the private data
#if (AWVER >= 0x300)
    LPSTR       name_ptr;           // name of the object or NULL
#endif
} APWC_POSTPB, FAR *APWC_POSTPB_PTR;


// ------------------------------------------------------------------------
//
// APWC_SENDPB - Parameter block for APWC_SEND message.
//
// ------------------------------------------------------------------------

typedef struct {
    DWORD   id;                 // object id of message recipient
    WPARAM  wparam;             // data to send to object
    LPARAM  lparam;             // data to send to object
} APWC_SENDPB, FAR *APWC_SENDPB_PTR;


// ------------------------------------------------------------------------
//
// APWC_INVALPB - Parameter block for APWC_INVAL message.
//
// ------------------------------------------------------------------------

#define INVAL_BACKWARD  1       // values for inval_type field
#define INVAL_FORWARD   2

typedef struct  {
    DWORD   id;                 // id of object to inval
    int     inval_type;         // INVAL_BACKWARD or INVAL_FORWARD
    RECT    rect;               // rectangle to invalidate
} APWC_INVALPB, FAR *APWC_INVALPB_PTR;


// ------------------------------------------------------------------------
//
// Flags for APWC_SETPAL message (may be combined):
//
// ------------------------------------------------------------------------

#define SP_DEFAULT  0           // no special treatment
#define SP_NOSTATIC 1           // force "no static" palette usage
#define SP_RAW      2           // use palette exactly as passed


// ------------------------------------------------------------------------
//
// APWC_EXPRPB - Describes Authorware values. Used with several messages.
//
// ------------------------------------------------------------------------

typedef int APW_EV_TYPE;

#define APW_STR_TYPE    0       // string
#define APW_LONG_TYPE   1       // long
#define APW_REAL_TYPE   2       // double
#define APW_ANY_TYPE    3       // native type of the expression 

typedef struct {
    APW_EV_TYPE type;           // type requested and returned
    union {
        HANDLE  string_value;   // result if APW_STR_TYPE (handle to string)
        double  real_value;     // result if APW_REAL_TYPE
        LONG    long_value;     // result if APW_LONG_TYPE
        LPSTR   string_ptr;     // string used only with APWC_SETVAR
    } value;
} APWC_EVALPB, FAR *APWC_EVALPB_PTR;


// ------------------------------------------------------------------------
//
// APWC_EXPRPB - Parameter block for APWC_EVALEXPR, APWC_COMPILE,
//  and APWC_EVAL.
//
// ------------------------------------------------------------------------

typedef struct {
    LPSTR   vis_form;           // the string to compile
    HANDLE  postfix_form;       // compiled form
    LONG    size;               // size of the postfix_form
    int     err_offset;         // error offset in the vis_form string 
    int     err_length;         // error length in the vis_form string
} APWC_COMPILEPB, FAR *APWC_COMPILEPB_PTR;

typedef struct {
    APWC_COMPILEPB  compile;        // compile arguments/results
    APWC_EVALPB     eval;           // eval arguments/results
} APWC_EXPRPB, FAR *APWC_EXPRPB_PTR;
        

// ------------------------------------------------------------------------
//
//  APWC_VARPB - Parameter block for APWC_GETVAR & APWC_SETVAR.
//
// ------------------------------------------------------------------------

#if (AWVER >= 0x300)

#define SV_DONTCREATE   0       // values for flags field of APWC_VARPB
#define SV_CREATE       1

typedef struct {
    LPSTR       var_name;       // the variable name
    LPSTR       at_icon_name;   // the @-reference icon name or NULL
    long        at_icon_id;     // the @-reference icon id or 0
    UINT        flags;          // create the variable if it doesn't exist?
    APWC_EVALPB eval;           // variable type and value
} APWC_VARPB, FAR *APWC_VARPB_PTR;

#endif


// ------------------------------------------------------------------------
//
// APW Control Messages
//
// These messages can be sent to the Authorware Presentation Window
// (via SendMessage) to carry out specific tasks. The description of
// each message includes the SendMessage arguments. Please see 
// USERCODE.WRI for complete details.
//
// ------------------------------------------------------------------------

typedef UINT APW_CONTROL_MESSAGE;

#define APWC_POST   0x6000
        //
        // Post a new object.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_POSTPB_PTR
        //                                      
        // Returns:
        //      >0  success, the unique object id
        //       0  cannot load DLL containing object procedure
        //      -1  cannot find object procedure in DLL
        //      -2  insufficient memory
        //      -3  error in parameter block

#define APWC_UNPOST 0x6001
        //
        // Destroy a posted object.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - id of the object to destroy
        //
        // Returns: 0

#define APWC_SEND   0x6002
        //
        // Send a message to a posted object.
        //  
        // hWnd     - handle to the APW presentation window
        // wParam   - the message to send
        // lParam   - APWC_SENDPB_PTR
        //
        // Returns:
        // The value returned by the object procedure.

#define APWC_INVAL  0x6003
        //
        // Invalidate a posted object so it will repaint.
        //  
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_INVALPB_PTR
        //
        //  Returns: 0

#define APWC_GETPAL 0x6004
        //
        // Get the current palette.
        // 
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - 0 (not used)
        //
        // Returns: An HPALETTE or NULL if unsuccessful.

#define APWC_SETPAL 0x6005
        //
        // Set the current palette.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - HPALETTE or NULL to use the course palette
        // lParam   - flags (the latter two may be combined with bitwise or)
        //
        // Returns:
        //  1   The palette was set. APW will delete the palette 
        //      when it's done with it.
        //  0   The palette was not set. The UCD must delete it.

#define APWC_RESETPAL   0x6006
        //
        // Reset the palette to the default course palette.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - 0 (not used)
        //
        // Returns: 0

#define APWC_COMPILE    0x6007
        //
        // Compile an expression.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_EXPRPB_PTR
        //
        // Returns: 0 if ok, else an EV_ERR code.

#define APWC_EVAL   0x6008
        //
        // Evaluate an expression compiled via APWC_COMPILE.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_EXPRPB_PTR
        //
        // Returns: 0 if ok, else an EV_ERR code.

#define APWC_EXPREVAL   0x6009
        //
        // Compile and evaluate an expression in one step.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_EXPRPB_PTR
        //
        // Returns: 0 if ok, else an EV_ERR code.

#define APWC_SETAUTOERASE   0x600a
        //
        // Set the automatic erase of posted objects for an icon.
        // This was called APWC_NOAUTOERASE prior to AW 3.0.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 to suppress auto-erase; 1 to enable auto-erase.
        // lParam   - icon id of the posted object
        //
        // Returns:
        //     0 if successful
        //    -1 if invalid icon id

#if (AWVER >= 0x300)

#define APWC_GETVAR 0x600b
        //
        //  Get the value of an Authorware variable.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_VARPB_PTR
        //
        // Returns: 0 if ok, else an EV_ERR code.

#define APWC_SETVAR 0x600c
        //
        // Set the value of an Authorware variable.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - APWC_VARPB_PTR
        //
        // Returns: 0 if ok, else an EV_ERR code.

#endif

#if (AWVER >= 0x301)
#define APWC_CLOSESOUNDDRIVER   0x600d
        //
        // Authorware holds the sound driver open for an amount of time after a 
        // sound icon plays. Each time a sound is played on Windows (on the Macintosh,
        // too), the sound driver needs to be opened and prepared for playing which 
        // takes a small amount of time. By holding the sound driver open, we prevent
        // this from affecting performance. This message closes the driver and stops
        // the sound.
        //
        // hWnd     - handle to the APW presentation window
        // wParam   - 0 (not used)
        // lParam   - 0 (not used)
        //
        // Returns: No return value.
#endif


// Define the old name of APWC_SETAUTOERASE for compatability.
#define APWC_NOAUTOERASE APWC_SETAUTOERASE


// ------------------------------------------------------------------------
//
//  APWN_AB - Argument block for all object notification messages. 
//
//  A pointer to this structure is passed as the first parameter
// to the object handler.
//
// ------------------------------------------------------------------------

typedef struct  {
    UINT    size;               // size of this structure
    DWORD   id;                 // unique id of the posted object
    int     icon_id;            // icon id of the posted object
    HWND    display;            // handle to the window the object is posted on
    WPARAM  wparam;             // args specific to the message
    LPARAM  lparam;             // args specific to the message
    RECT    rect;               // bounding rectangle of the object
    LPVOID  data_ptr;           // pointer to the private data area
    UINT    data_size;          // size of the private data area
#if (AWVER >= 0x300)
    LPSTR   name_ptr;           // pointer to object name
    BOOL    authoring;          // TRUE in authoring system, FALSE in runtime
#endif
} APWN_AB, FAR *APWN_AB_PTR;


// ------------------------------------------------------------------------
//
//  APWN_PROP - Parameter block for APWN_SETPROPERTY and APWN_GETPROPRTY.
//
// ------------------------------------------------------------------------

#if (AWVER >= 0x300)

typedef struct {
    LPSTR   property;           // property string
    LPSTR   value;              // value string for APWN_SETPROPERTY
} APWN_PROP, FAR *APWN_PROP_PTR;

#endif


// ------------------------------------------------------------------------
//
// Object Notification Messages
//
// These messages are sent to the object procedure of objects created via
// the APWC_POST message. The first argument to the object procedure is
// a pointer to an APWN_AB argument block. The contents of the wparam and 
// lparam fields of the argument block are object-specific and are
// described below with each message.
//
// The object procedure must return 0 for any message that it does not
// explicity handle.
//
// Please see USERCODE.WRI for complete details.
//
// ------------------------------------------------------------------------

typedef unsigned short APW_NOTIFICATION_MESSAGE;

#define APWN_INIT   0
        //
        // Sent when the object is initially created.
        //
        // wparam   - 0 (not used)
        // lparam   - 0 (not used)
        //
        // Return 0
                
#define APWN_DESTROY    1
        //
        // Sent when the object is being destroyed. The object procedure 
        // should free any memory it allocated.
        //
        // wparam   - 0 (not used)
        // lparam   - 0 (not used)
        //
        // Return 0

#define APWN_PAINT  2
        //
        // Sent when the object needs to be repainted. The clipping region
        // of the device context is set to the intersection of the invalid
        // region and the bounding rectangle of the object.
        //
        // wparam   - the device context (HDC) to paint into
        // lparam   - 0 (not used)
        //
        // Return 0

#define APWN_SAVE   3
        //
        // Sent when Authorware is saving the state of a running file so it 
        // can resume execution later. The object's private data is saved 
        // automatically.  If there is any additional state information that 
        // the object needs to save it should return a global handle containing
        // the additional state information. Authorware will save the contents 
        // of the global handle and free it.
        //
        // wparam   - 0 (not used)
        // lparam   - 0 (not used)
        //
        // Return a global handle (HGLOBAL) to extra data or NULL if there
        // is no extra data to save.
        //

#define APWN_RESTORE    4
        //
        // Sent when Authorware is resuming a file. The object should restore
        // its state as necessary to resume execution. The object's private
        // data is in the same state as it was when the APWN_SAVE message
        // was sent. If the object responded to APWN_SAVE by returning a 
        // handle to additional save data, then the wparam field contains
        // a handle to that data. The object procedure must free the handle.
        //
        // wparam   - handle to saved data (HGLOBAL) or NULL if none was saved.
        // lparam   - 0 (not used)
        //
        // Return 0

#define APWN_LBDOWN 5
        //
        // Sent when the left mouse button is clicked while the mouse is
        // within the bounding rectangle of the object.
        //
        // wparam   - 0 (not used)
        // lparam   - click point, x in loword, y in hiword
        //
        //  Return:
        //      1   to get mouse capture and the subsequent APWN_LBUP message.
        //      0   if you don't care about the mouse up.

#define APWN_LBDBLCK    6
        //
        // Sent when the left mouse button is double-clicked while the mouse 
        // is within the bounding rectangle of the object.
        //
        // wparam   - 0 (not used)
        // lparam   - click point, x in loword, y in hiword
        //
        //  Return:
        //      1   to get mouse capture and the subsequent APWN_LBUP message.
        //      0   if you don't care about the mouse up.

#define APWN_LBUP   7
        //
        // Sent when the left mouse button is released. This message is only
        // sent to the object if it returned 1 in response to a APWN_LBDOWN
        // message or a APWN_LBDBLCK message.
        //
        // wparam   - 0 (not used)
        // lparam   - click point, x in loword, y in hiword
        //
        // Return 0

#define APWN_WINDOWOPEN 8
        //
        // Sent when the Authorware presentation window is opening. The
        // handle of the presentation window will be different than it
        // was when the window was closed.
        //
        // wparam   - 0 (not used)
        // lparam   - 0 (not used)
        //
        // Return 0

#define APWN_WINDOWCLOSE    9
        //
        // Sent when the Authorware presentation window is closing. Note that
        // objects are not destroyed when the presentation window closes since
        // the window may be opened and closed many times during authoring
        // without restarting execution of the file. This message usually
        // only matters with objects that are directly dependent on the
        // presentation window in some way. (eg. They may create child windows
        // of the presentation window that will be destroyed when the
        // presentation window is destroyed.)
        //
        // wparam   - 0 (not used)
        // lparam   - 0 (not used)
        //
        // Return 0

#if (AWVER >= 0x300)

#define APWN_SETPROPERTY    10
        //
        // Sent when the author calls the SetProperty() system function.
        // This message is sent to the object whose name matches the
        // first parameter of the SetProperty() function. It is up to
        // the object to interpret what it means to set a property.
        //
        // wparam   - 0 (not used)
        // lparam   - APWN_PROP_PTR
        //
        // Return 0.

#define APWN_GETPROPERTY    11
        //
        // Sent when the author calls the GetProperty() system function.
        // This message is sent to the object whose name matches the
        // first parameter of the GetProperty() function. 
        //
        // wparam   - 0 (not used)
        // lparam   - APWN_PROP_PTR
        //
        // Return a global handle to a zero-terminated string or NULL.

#define APWN_CLOSEWINDOW    12
        //
        // Sent when the author calls the CloseWindow() system function.
        // This message is sent to the object whose name matches the
        // first (and only) parameter of the CloseWindow() function. 
        //
        // wparam   - 0 (not used)
        // lparam   - 0 (not used)
        //
        // Return:
        //   1  If you handled the message. Authorware does nothing.
        //   0  If you want Authorware to destroy the object.

#define APWN_MOUSEMOVE  13
        //
        // Sent when the mouse moves while the object has mouse capture. 
        // This message is only sent to the object if it returned 1 in 
        // response to a APWN_LBDOWN or APWN_LBDBLCK message.
        //
        // wparam   - 0 (not used)
        // lparam   - mouse point, x in loword, y in hiword
        //
        // Return 0
        
#define APWN_HITTEST    14
        //
        // Sent when Authorware needs to determine if a point lies on
        // the object. Usually associated with a mouse click. Authorware
        // only sends this message if the point lies within the bounding
        // rectangle of the object. By responding to the message the
        // object can implement transparent areas.
        //
        // wparam   - 0 (not used)
        // lparam   - mouse point, x in loword, y in hiword
        //
        //  Return:
        //      0   If the point hits the object.
        //      1   If the point doesn't hit the object.

#endif

#define APWN_USER   10000
        //
        //  Any custom notification messages should start here.


// ------------------------------------------------------------------------
//
// APW_PROC - A pointer to an object handler.
//
// ------------------------------------------------------------------------

typedef long (FAR PASCAL *APW_PROC)(APWN_AB_PTR, APW_NOTIFICATION_MESSAGE);


#if defined(_WINDOWS) && !defined(RC_INVOKED)
	#if (_MSC_VER >= 900)
		#pragma pack(pop)
	#else
		#pragma pack()
	#endif
#endif // !def RC_INVOKED


#endif // _APWPOST_
